### This script contains an installation command for packages that
# are not installed  when you download R from the CRAN website.
# It also contains functions that I have not found in the base packages
# but that I have found useful for many  analyses . The simplest approach
# is to open this script in R, then in the Edit menu, select "Run all."
# If you then type "ls()" , you should find the function names listed.


#####             install packages
#  When R is installed, several packages (collections of functions, 
#  and related material such as data sets, examples, and descriptions
#  are also installed. In addition, we have used functions from other  containing 
#  packages. The following command will install these packages:
	install.packages(c("plotrix","moments","pwr","car","gmodels",
	    "multcomp","lme4","languageR","mlogit","nortest","psych","polycor",
	    "mvtnorm","lmtest","MBESS","gsl","effects","MASS","rms"))

	
#####################################################################

## Kellerman's functions
# Kellerman's RProfile script may be found at
# "http://psych-swiki.colorado.edu:8080/LearnR/uploads/10/Rprofile.3.site"
# Of the functions in that file, the following two are especially useful.

##DIMX
dimx<-function (dd)
{if (is.null(dim(dd))) length(dd) else dim(dd)}

##CLASSX
classx<-function (dd)
{if (length(class(dd))>1) class(dd)[1] else class(dd)}

## LS lists all objects and their classes that are in the workspace
LS<-function (space=1,pattern = "")
{

if (length(ls(space,pat=pattern)) <1){stop("No objects are in memory")}

obs <- ls(space, pat = pattern)
cat(
    formatC("DIM/LEN"),
    formatC("NAME", width=max(nchar(obs)+5)),
    formatC("CLASS",width=16),
    "\n")

if (length(ls(space,pat="tmp")) >0){
  
 for (i in 2:length(obs)) {
   widim <- 0
   ww <- eval(parse(t = paste("length(dimx(", obs[i], "))")))
   for (k in 1:ww){
     widim <- eval(parse(t = paste("length(dimx(",obs[i],"))+widim-2+nchar(dimx(", obs[i], ")[k])")))}
                       
   cat(
            eval(parse(t = paste("dimx(", obs[i], ")"))),
            formatC(obs[i], width=max(nchar(obs))-widim+10),
            formatC(eval(parse(t = paste("class(", obs[i], ")"))),1, 16), 
            "\n")  }}

else{
   for (i in 1:length(obs)) {
   widim <- 0
   ww <- eval(parse(t = paste("length(dimx(", obs[i], "))")))
   for (k in 1:ww){
     widim <- eval(parse(t = paste("length(dimx(",obs[i],"))+widim-2+nchar(dimx(", obs[i], ")[k])")))}
                       
   cat(
            eval(parse(t = paste("dimx(", obs[i], ")"))),
            formatC(obs[i], width=max(nchar(obs))-widim+10),
            formatC(eval(parse(t = paste("class(", obs[i], ")"))),1, 16), 
            "\n")  }}}

## INFO  gives an objects mode, class, variable names, and dimensions 
# or length 

info <- function(x){
cat("MODE:          ")
cat(mode(x))
cat("\n")
cat("CLASS:         ")
cat(class(x))
cat("\n")
cat("DIM or LENGTH: ")
cat(dimx(x))
cat("\n")
cat("NAMES:         ")
if (length(names(x))>6) cat(names(x)[1:6]," ...")
else cat(names(x))
cat("\n")}

 
##### The following are additional useful functions
                           Session 4 
### dbin_table
## A function to generate binomial tables for any x and p
# x= number(s) of successes;  size=number of trials
# nr = number of items in x, p = p(success)
# The output is a table of p(x successes in n trials); n=size
   dbin_table=function(x,size, nr, p) {
      j=1
      prob=matrix(0, nr, length(p)) # length(p)= number of probabilities
	colnames(prob)=p
	rownames(prob)=x
      while(j<=length(p)) {
         prob[,j]=dbinom(x,size,p[j])
         j=j+1
      }
   return(round(prob,3))
    }

#####################################################################

 #####                      Session 5
# The following sort function is very useful. Sorts the entire table
# with reference to a designated column
   sort.data.frame = function(x, key, ...) {
      if (missing(key)) {
         rn = rownames(x)
      if (all(rn %in% 1:nrow(x))) rn = as.numeric(rn)
         x[order(rn, ...), , drop=FALSE]
      } else {
        x[do.call("order", c(x[key], ...)), , drop=FALSE]
   }
}


#####
# This function reshapes a matrix; for example, if the matrix is
# 4 x 5, we can change it to 2 x 10
	reshape=function(x1,nrow,ncol) {
    		x2=t(x1)
    		dim(x2)=c(ncol,nrow)  #dimensions of x3
    		x3=t(x2)
 	}

#####################################################################

#####                           Session 6/7
# Smithson has a script with four functions that provide limits
# on delta, the noncentrality parameter of the t distribution. 
#I have extended this to provide limits on Cohen's d as well. Although there are 
# four functions here, "lot" and "hit" are the ones you will use directly to get linits
# on delta and d. However, these call on "ptnoncent" so it should be installed 
# in your workspace. The fourth function is used in power calculations and calls
# on "hit".
#
     ptnoncent <- function(tx, df, nonc = 0, itrmax = 100, errmax = 1E-6)
{
	if(min(df) <= 0)
		stop("All df must be > 0")
	lengths <- c(length(tx), length(df), length(nonc))
	if(any(lengths < (ltx <- max(lengths)))) {
		tx <- rep(tx, length.out = ltx)
		df <- rep(df, length.out = ltx)
		nonc <- rep(nonc, length.out = ltx)
	}
	tnc <- numeric(ltx)
	del <- nonc
	negdel <- (tx < 0)
	del <- ifelse(negdel,  - del, del)
	xx <- (tx * tx)/(tx * tx + df)
	lambda <- del * del
	p <- 0.5 * exp(-0.5 * lambda)
	q <- 0.79788456080286496 * p * del
	ss <- 0.5 - p
	a <- rep(0.5, ltx)
	b <- 0.5 * df
	rxb <- (1 - xx)^b
	albeta <- 0.57236494292469997 + lgamma(b) - lgamma(
		a + b)
	xodd <- pbeta(xx, a, b)
	godd <- 2 * rxb * exp(a * log(xx) - albeta)
	xeven <- 1 - rxb
	geven <- b * xx * rxb
	tnc <- p * xodd + q * xeven
	itr <- 0
	err <- rep(1, ltx)
	while((itr <- itr + 1) <= itrmax && max(err) > errmax
		) {
		a <- a + 1
		xodd <- xodd - godd
		xeven <- xeven - geven
		godd <- (godd * xx * (a + b - 1))/a
		geven <- (geven * xx * (a + b - 0.5))/(a +
			0.5)
		p <- (p * lambda)/(2 * itr)
		q <- (q * lambda)/(2 * itr + 1)
		ss <- ss - p
		tnc <- tnc + p * xodd + q * xeven
		err <- 2 * ss * (xodd - godd)
	}
	if(itr > itrmax)
		warning("maximum number of iteration reached"
			)
	tnc <- tnc + 1 - pnorm(del)
	ifelse(negdel, 1 - tnc, tnc)
}

#######
#CIs and power for the noncentral t noncentrality parameter (ncp):
#The function "lot" computes the lower CI limit and "hit" computes the upper limit.
#Both functions take 4 arguments: observed t-value, df, confidence level, and
#  number of samples (g=1 for one sample,2 for two independent groups). 
# NOTE: if the single sample contains correlated scores, the d limits are
# actually limits on Cohen's dz. See Cohen (1988) for further details about d and dz.
#NOTE 2: conversion of delta limits to d limits assumes equal n if g = 2.
#The function powt uses hit to find the ncp, given df1, alpha, and power level.


lot <- function(tval,df,conf,g) {
#g=1 or 2 depending on whether 1 sample (e.g., correlated scores) or 2 samples.
	ulim <- 1 - (1-conf)/2
#	This first part finds a lower and upper value from which to start.
	lc <- c(-tval,tval/2,tval)
	while(ptnoncent(tval,df,lc[1])<ulim) {
		lc <- c(lc[1]-tval,lc[1],lc[3])
	}
#	This next part finds the lower limit for the ncp.
	diff <- 1
	while(diff > .00001) {
	if(ptnoncent(tval,df,lc[2])<ulim) 
			lc <- c(lc[1],(lc[1]+lc[2])/2,lc[2]) 
			else lc <- c(lc[2],(lc[2]+lc[3])/2,lc[3])
		diff <- abs(ptnoncent(tval,df,lc[2]) - ulim)
		ucdf <- ptnoncent(tval,df,lc[2])
	}
      if (g==1) {
         lcd=lc[2]/sqrt(df+1)
       } else {
         lcd = lc[2]*sqrt(4/(df+2))
        }
       labels = " low_delta     low_d   hi_p"
       stats = c(lc[2],lcd,ucdf)
       print(labels)
       print(stats)
}

hit <-
function(tval,df,conf,g)
{
	llim <- (1-conf)/2
#	This first part finds an upper value from which to start.
	uc <- c(tval,1.5*tval,2*tval)
	while(ptnoncent(tval,df,uc[3])>llim) {
		uc <- c(uc[1],uc[3],uc[3]+tval)
	}
#	This next part finds the upper limit for the ncp.
	diff <- 1
	while(diff > .00001) {
	if(ptnoncent(tval,df,uc[2])<llim) 
			uc <- c(uc[1],(uc[1]+uc[2])/2,uc[2]) 
			else uc <- c(uc[2],(uc[2]+uc[3])/2,uc[3])
		diff <- abs(ptnoncent(tval,df,uc[2]) - llim)
		lcdf <- ptnoncent(tval,df,uc[2])
	}
      if (g==1) {
         ucd=uc[2]/sqrt(df+1)
       } else {
         ucd = uc[2]*sqrt(4/(df+2))
        }
      labels = " hi_delta     hi_d     lo_p"
       stats = c(uc[2],ucd,lcdf)			
       print(labels)
       print(stats)

}
#
powt <-
function(df,alpha,power)
{
	hit(qt(1-alpha,df),df,1-(1-power)*2)
}



#### The following function provides the trimmed t test
# The parameters are the group names,n1 and n2,the trim from each tail,
#   the number trimmed from each tail, and the quantiles for the trim
   trimmed.t=function(x1, x2,n1,n2,trim, k, probs=c(p1,p2)){
      m1.trim = mean(x1,trim)
      m2.trim = mean(x2,trim)
      m_diff = m2.trim - m1.trim
#First deal with Group 1
      xq1=quantile(x1,probs=probs)
#xq1 are the scores at the p1 and p2 quantiles
      x1[x1< xq1[1]]= xq1[1]
# all x1 < xq1[1] are set equal to xq1[1]
      x1[x1>xq1[2]] = xq1[2]
# all x1 > xq1[2] are set equal to xq[2]
      ss1 = (n1-1)*var(x1) #sum of squares for group 1
# Now do the same for Group 2
      xq2=quantile(x2,probs=probs)
      x2[x2< xq2[1]]= xq2[1]
      x2[x2>xq2[2]] = xq2[2]
      ss2 = (n2-1)*var(x2)
      ssw = ss1+ss2    #winsorized within-groups sum of squares
      s_pool = sqrt(ssw/(n1+n2-2-4*k))
      se_win = s_pool*sqrt((1/(n1-k))+(1/(n2-k)))
#winsorized standard error of the difference in means
      t_trim = m_diff/se_win
      df=n1+n2-2-4*k
      p_trim_t = pt(t_trim, df, lower.tail = FALSE)
      labels = "trim t df p_value m1.trim m2.trim se_win"
      print(labels)
      stats = round(c(t_trim, df, p_trim_t, m1.trim, m2.trim, se_win),3)
#the trimmed t and its p value
      print(stats)
   }

#####################################################################

#####                  Session 8   

## f_trim  
# The function for the trimmed F test
# Assumes equal n
# x is a data frame -- check class(x)
# trim is fraction trimmed from each tail
# a is number of groups
# k  is number of scores trimmed from each tail
# Note -- if quantiles are not integer, discard fraction to decide on k
#	E.g., n = 13. Then R has 20% quantiles at 3.4, 10.6, k = 3 because
#      there are 3 scores below 3.4, 3 above 10.6
# p1,p2 are trim, 1-trim -- usually .2,.8

   f_trim=function(x,trim, k, probs=c(p1,p2)){
	m.trim = rep(0,ncol(x))
	for (i in 1:ncol(x)){
		m.trim[i] = mean(x[,i],trim=trim)
	}	
	a=ncol(x)
	n=nrow(x)
	j=1
	while(j<=a){
		xq=quantile(x[,j],probs=probs)
		y = x[,j]
		y[y<xq[1]] = xq[1]
		y[y>xq[2]] = xq[2]
		x[,j]=y
		j=j+1
	}
	msb_tr = (n-2*k)*sd(m.trim)^2 
# MS(between groups) for trimmed means
	v_x = sapply(x,sd)^2
	ssw_win = (n-1)*mean(v_x)  #Winsorized SS
	msw_win = ssw_win/(a*(n-1-2*k))
	f_tr = round(msb_tr/msw_win,3)
	dfb = a-1
	dfw=a*(n-1-2*k)
	p_value=pf(f_tr,dfb,dfw,lower.tail=FALSE)
	labels = "  F     df1      df2      sig"
	stats = round(c(f_tr, dfb, dfw, p_value),3)
	print(labels)
	print(stats)
   }

#####################################################################

#####                  Session 10

## A function to test contrasts involving unequal n and unequal variances
# Each row of w is a set of contrast coefficients; x is a dependent measure, 
# and f is anindependent variable. See the Session 10 script for an example of 
# the application
  Welch.t=function (w,x = "data vector", f = "factor vector") {
    if (class(f) != "factor") {
        f = factor(f)
    }
    m=as.vector(tapply(x,f,mean))
     v=as.vector(tapply(x,f,var))
     n=as.vector(tapply(x,f,length))
     psi=w2%*%m        # %*% indicates matrix multiplication
     se_psi=sqrt(colSums( t(w2^2)*v/n))
#Std. Error of the contrast (psi). 
     df_den=colSums( t(w2^4)*(v^2)/((n^2)*(n-1)))    #df denominator
     df = (se_psi^4)/df_den 
     t = abs(psi/se_psi)
     p_value = 2*pt(t,df,lower.tail=FALSE)
     t.crit=qt(.025,df,lower.tail =FALSE)  
     LB_CI = psi-se_psi*t.crit       #The lower coonfidence interval bound
     UB_CI = psi+se_psi*t.crit
     Welch.out=data.frame(round(cbind(psi, df, 
      p_value, se_psi,t, LB_CI,UB_CI),4))
     colnames(Welch.out)=c("PSI", "df","p value","std. error","t","LB","UB")
      return(Welch.out)
}


### A function to perform the Games-Howell test of contrasts when variances
# and n's are not equal
	Games_Howell=function (x = "data vector", f = "factor vector") {
    		if (class(f) != "factor") {
        	f = factor(f)
      }
      k = length(levels(f))     #Number of levels of factor
     levels.n = levels(f)         #Names of levels
    m=as.vector(tapply(x,f,mean))
     v=as.vector(tapply(x,f,var))
     n=as.vector(tapply(x,f,length))
      df = n-1
     ndiff=gamma(k+1)/(2*gamma(k-1))
    q_tky = rep(0,ndiff)
     p_value= rep(0,ndiff)
     diff = rep(0,ndiff)
     se = rep(0,ndiff)                #standard errors
     row.n=rep(0,ndiff)             #row names for output
     dfprime = rep(0,ndiff)        # corrected df
     mm = 1
     i = 1
     while (i <= k-1){
     j = i+1
   while (j <= k){
         diff[mm]=m[j]-m[i]
         se[mm] = sqrt(v[j]/n[j] + v[i]/n[i])
         q_tky[mm]  = sqrt(2)*abs(diff[mm]/se[mm])
#q_tky is the calculated q statistic
         dfprime[mm]=(se[mm]^4)/((v[j]^2)/((n[j]^2)*df[j]) + 
         (v[i]^2)/((n[i]^2)*df[i]))
         p_value[mm] = ptukey(q_tky[mm],k,dfprime[mm],
         lower.tail=FALSE) 
         row.n[mm]=paste(levels.n[j],levels.n[i],sep="-")
# row.n is the set of row names 
         mm = mm+1
     j=j+1
     }
     i = i+1
    }
    q.crit=qtukey(.05,4,dfprime,lower.tail=FALSE)  
# q.crit is the critical studentized range value
     LB_CI = diff-se*q.crit/sqrt(2) #The lower coonfidence interval bound
     UB_CI = diff+se*q.crit/sqrt(2)
     stats.out=data.frame(round(cbind(diff, dfprime, 
      p_value, se, LB_CI,UB_CI),4))
      GH.out = cbind(row.n,stats.out)
     return(GH.out)
}

#####################################################################

#####                  Session 13
### The following function simulates ANOVA and ANCOVA, calculates 
#  relative efficiency of the two analyses of the same data sets.
## a= no. of groups, n = group size,r = X-Y correlation
   RE_sim=function(rseed,nsim,a,n,r){
    N=a*n
    A = as.factor(rep(c(1:a),n))
     ems1=0       #EMS(error) for ANOVA
     p1 = 0       #p value for ANOVA
     ems2 = 0     #EMS(error) for ANCOVA
     p2 = 0       #p value for ANCOVA
     set.seed = rseed
    j = 1
     while(j<=nsim) {
        zx = rnorm(N)
        zy=r*zx + rnorm(N)
        X = round(50+15*zx)
        Y = round(50+15*zy)+rep(c(10,0,-10),n)
# First do the ANOVA  (ems1)
        table1=anova(aov(Y~A)) 
        ems1=ems1+table1[2,3]/nsim
        p1=p1+table1[1,5]/nsim
#  Now ANCOVA      (ems2)
         AX.model=aov(Y~A+X)
         X.model=aov(Y~X)
         table2 = anova(X.model,AX.model)
         ems2=ems2+(table2[2,2]/table2[2,1])/nsim
         p2=p2+table2[2,6]/nsim
         j = j+1
     }
      RelEff = ems1/ems2
     head1 = "    ANOVA            ANCOVA"
     head2 = " EMS    aver(p)   EMS    aver(p)     RelEff"
     stats = round(c(ems1,p1,ems2,p2,RelEff),3)
     print(head1)
     print(head2)
     print(stats)
     }
 

### A function to create a Latin square
# This function is a slight modification of Bret Larget's function at
#  http://www.stat.wisc.edu/courses/st572-larget/Spring2007/handouts17-4.pdf 
     Latin = function (n,nrand)  {
     x = t(matrix(LETTERS[1:n], n, n))
     for (i in 2:n) x[i, ] = x[i, c(i:n, 1:(i - 1))]
     for (i in 1:nrand) {
       x = x[sample(n), ]
       x = x[, sample(n)]
     }
     LSq<<-as.data.frame(x) # "<<- " makes the object globally available
      print(LSq)
      }

### Tukey's test of nonadditivity
# requires x (a data frame) and the error sum of squares from an ANOVA
# This function assumes a balanced (no missing scores) design,
# and uses the usual subjects x treatments layout rather than a stack.
  nonadd.test=function(x,SSerror){
     x.mat=as.matrix(x)
     cmean=colMeans(x.mat)
     rmean=rowMeans(x.mat)
     gm =mean(cmean)
     cdev=cmean-gm
     rdev=rmean-gm
     cp=x.mat%*%cdev
     N=t(rdev)%*%cp
     D=sum(cdev^2)*sum(rdev^2)
     SSnonadd=(N^2)/D
     SSbal=SSerror-SSnonadd
     df=(dim(x.mat)[1]-1)*(dim(x.mat)[2]-1)-1
     MSbal = SSbal/df
     Fnonadd = SSnonadd/MSbal
     p_value=1-pf(Fnonadd,1,df)
     print("   F   p")
     stats = c(Fnonadd,p_value)
     print(stats)
     p.trans=1-(N/D)*gm
     print("power for transformation is")
     print(p.trans)
   }

### Cochran's Q test for 0/1 data
#  See http://tolstoy.newcastle.edu.au/R/e2/help/06/09/0974.html
cochranq.test <- function(mat)
{
  k <- ncol(mat)

  C <- sum(colSums(mat) ^ 2)
  R <- sum(rowSums(mat) ^ 2)
  T <- sum(rowSums(mat))


  num <- (k - 1) * ((k * C) - (T ^ 2))
  den <- (k * T) - R

  Q <- num / den

  df <- k - 1
  names(df) <- "df"
  names(Q) <- "Cochran's Q"

  p.val <- pchisq(Q, df, lower = FALSE)

  QVAL <- list(statistic = Q, parameter = df, p.value = p.val,

               method = "Cochran's Q Test for Dependent Samples",
               data.name = deparse(substitute(mat)))

  class(QVAL) <- "htest"
  return(QVAL)
} 
#####################################################################

#####          Session 19

### Test of partial correlations
# k = number of variabled partialled out
# rho is null-hypothesis value
# r = partial r
# confidence = 1-alfa
  partial.r.test=function(n,k,rho,r,alfa) {
  if (rho==0) {
     stat = sqrt((n-2-k)/(1-r^2))
     sig=2*pt(stat,n-2-k,lower.tail=F)   # two-tailed
  } else {
     stat= (fisherz(r)-fisherz(rho))*sqrt(n-3-k)
     sig=2*pnorm(stat,lower.tail=F)}
 # confidence interval
   LB_r=fisherz(r)-qnorm(alfa/2,lower.tail=F)*sqrt(1/(n-3-k))
   UB_r=fisherz(r)+qnorm(alfa/2,lower.tail=F)*sqrt(1/(n-3-k))
   labels = " Statistic  p-value   Upper Bound   Lower Pound"  
   stats = round(c(stat,sig,UB_r,LB_r),5)
   print(labels)
   print(stats)
 }

#####################################################################

#####	      	Session 20
###  This tests the difference between two independent slopes
# Assumes 2 data frames, no cases with missing X or Y values
b.diff.test=function(data.1,data.2,x.1,x.2,y.1,y.2) {
     model.1 = lm(y.1~x.1)
     model.2 = lm(y.2~x.2)
     sum.1= summary(model.1)$coefficients
     sum.2= summary(model.2)$coefficients
     b1.1=sum.1[2,1]	      # slope for data.1
     b1.2=sum.2[2,1]	      # slope for data.2
     c(b1.1,b1.2)	      # print the slopes

# The standard errors of the two slopes can  be obtained from
     df.1 = length(data.1[,1])-2   # degrees of freedom 
     df.2 = length(data.2[,1])-2   
     ss.1 = (df.1+1)*var(x.1)    # SS(X|data.1)   
     ss.2 = (df.2+1)*var(x.2)    # SS(X|data.2) 
    sse.1 = sum(residuals(model.1)^2) # SS(error|data.1)
    sse.2 = sum(residuals(model.2)^2) #SS(error|data.2)
    se=sqrt((sse.1+sse.2)/(df.1+df.2)) # pooled standard error
 
#  Therefore,
     se.diff = se*sqrt(1/ss.1 + 1/ss.2)
     t = (b1.2-b1.1)/se.diff
     p_t = 2*pt(t,df.1+df.2,lower.tail=F)
     stats = round(c(b1.2, b1.1, se.diff, t, p_t),3)
     header=c("   b1.2  "	,"   b1.1  ",	 "  se.diff  ", "   t   ", " p-val ")	# two-tailed p value
    stats.out=data.frame(rbind(header,stats))
     return(stats.out)
}


#####################################################################

#####	      	Session 22
### Function for a confidence interval for a squared multiple R
# n = number of data points; p = number of predictors; R2  =  R-squared;
# alfa = 1-confidence level
R_sq.ci= function(R2,p,n,alfa) {
     SE.R2 =sqrt((4*R2)*((1-R2)^2)*((n-p-1)^2)/((n^2-1)*(n+3)))	# std, error of R-squared
     z = qnorm(1-alfa/2)
     upper = R2+z*SE.R2
     lower = R2-z*SE.R2
    CI = cbind(upper, lower)
    return(CI)
}

### Function for confidence interval for the difference between 2 squared Rs
# Large n approximation; see Algina & Keselman (1999, Psychological Methods,
# 4,76-83) for tables of minimum sample size needed to closely approximate the 
#  desired level of confidence
 R_sq_d.R2 = function(R2.1, R2.2,n1,n2,alfa) {
	SE_d.R2=sqrt((4/n1)*(R2.1)*((1-R2.1)^2) + (4/n2)*(R2.2)*((1-R2.2)^2))
     z = qnorm(1-alfa/2)
     upper = (R2.1-R2.2)+z*SE_d.R2
     lower = (R2.1-R2.2)-z*SE_d.R2
     CI = cbind(upper, lower)
     return(CI)
}
	


