
                  #####	 SCRIPTS for PART 3 

## NOTE: The "multcomp" package should have been installed previously.
# If you have not done so, do so now.

#####                 Session 13
##### 	Comparing Experimental Designs and Analyses


### The Treatment x Blocks Design
# Read in the "TxB example.csv" data file
	txb.data = read.csv(file.choose(),header=T,sep=",")
	head(txb.data)

# Now for the ANOVA, ignoring B and X
     A.aov=aov(Y~factor(A),data=txb.data)
     summary(A.aov)
# And now the treatment x blocks ANOVA
     AB.aov=aov(Y~factor(A)*factor(B),data=txb.data)
     summary(AB.aov)
# The A x B ANOVA yields a smaller p value. To test whether the 
# improvement is significant,
     anova(A.aov,AB.aov)
# The Sum of Sq reported is the difference between the error SS
# in the two analyses; call this SS_diff. Then the reported F =
# (SS_diff/df_diff)divided by the error mean square for the AB design.
# In this example, F = ((15348.2-6923.3)/(33-24))/288.14 = 3.25

## Exercise 13.1
# Write an R expression to calculate the relative efficiency of the
# two designs.

## Exercise 13.2
# Analyze the EX13_1 data ignoring the blocking variable, and then
# including the blocking variable. Then test whether the residual 
# error variances differ significantly.

### The Analysis of Covariance (ANCOVA)
# We analyze Y as a function of both A and X (X is the covariate) 
	AX.model=aov(Y~factor(A)+X,data=txb.data)
	summary(AX.model)
#  and again as a function only of X
	X.model=aov(Y~X,data=txb.data)
	summary(X.model)
 
# Then compare the models (this is essentially an analysis of covariance)  
     anova(X.model,AX.model)
#The resulting table is
#  Res.Df    RSS Df Sum of Sq      F  Pr(>F)  
#1     34 11481.6                               
#2     32  8150.1  2    3331.5 6.5403 0.004155 **

# In this table, the adjusted A sum of squares is the difference between
# the residuals in  the X.model and AX.model outputs; this is 3331.5. 
# The adusted error SS is 8150.1. Then the ANCOVA F test is 
# F = (3331.5/2)/(8150.1/32)  = 6.54; p = .000.
#  In summary, Using the X as a blocking variable 
# or as a covariate increases efficiency, providing a lower
# p value (provided there is a reasonable correlation with Y
# amd other assumptions are met.

## Exercise 13.3
# Perform the ANCOVA on the EX13_1 data 

#Let's clean up
      rm(A.aov,AB.aov,AX.model,X.model,txb.data)


### A Simulation of the ANOVA and ANCOVA Analyses
# Here, we illustrate how R can carry out a simulation.
# We run a number of simulated experiments, and compsre the 
# average mean squares and average p_values for a one-factor experiment
# with and without X as a covariate.
# A function ,"RE_sim",to simulate ANOVA and ANCOVA and calculate 
# relative efficiency (RelEff) is in the Functions.R script. The
# parameters are the seed, number of simulations. number of groups, 
# the group size, and the XY correlation. To test the function
    RE_sim(1306,200,3,12,.7)
# The output gives the average mean square and p value for each design
# and the ratio of the EMS (RelEff). We can see the advantage of the ANCOVA
# in this ratio.  	
      
## Exercise 13.4
# Write the commands to simulate 100 replications of a Treatments x
# Blocks experiment, and print the error EMS and average p_valuei
# Clean up the objects from this exercise using the rm function.


### Repeated=Measures Designs
# The following is an example of a one-factor repeateed-measures design.
     Beer=factor(rep(c(1:4),8))
     Subject = factor(rep(c(1:8),each=4))
     Rating = c(2,5,3,3,4,6,5,4,5,7,4,5,3,4,3,4,6,7,6,5,2,5,4,3,
       4,5,6,4,3,6,4,6)
     beer.ex = data.frame(cbind(Subject,Beer,Rating))
     head(beer.ex)
## NOTE: The data are arrayed with a column for each variable.

# Let's get the mean rating of each beer
     tapply(Rating,Beer,mean)

# The ANOVA: Method 1
# For one-factor repeated-measures designs
	beer1.aov=aov(Rating~Beer+Subject)
	summary(beer1.aov)
# Note that the error term is a residual, basically Beer x Subject
# We can get summary statistics following the ABOVA:
	model.tables(beer1.aov, type="mean", se=T)
# The S.E. of the diference assumes sphericity (homogeneity of variances
# of differences)
	
# The ANOVA: Method 2 
# This is useful in more complex designs:
     beer2.aov=aov(Rating~Beer+Error(Subject/Beer))	
     summary(beer2.aov)
# Note that the within-subject factor (Beer) appears after the slash.
# The model.tables function does not yield SE's with this approach


### Reformatting a Subjects x Treatments Array
# Typically, repeated-measures data are presented in a row for each subject.
# We can convert to this format using the unstack function.
 	beer.ex2 =  cbind(1:8,unstack(beer.ex,Rating~Beer))
     colnames(beer.ex2)= c("Subject","Beer1","Beer2","Beer3","Beer4")
     beer.ex2

# In Session 15, we will provide an example of a function that allows
# analysis of variance in the form exemplified by beer.ex2, However,
# the extended (or long) form is often easier to work with.
# To convert beer.ex2 to beer.ex3 (extended format),
     beer_1= data.frame(t(beer.ex2[,2:5]))  # t(x) is the transpose of x
     colnames(beer_1)=paste("S",1:8,sep="") 
# Note the paste function; sep="" indicates no space between S and number
     beer_2=stack(beer_1)  	#This is the key; converts to long form
     Beer=rep(c("Beer1","Beer2","Beer3","Beer4"),8)
     beer.ex3 = cbind(beer_2,Beer)
     colnames(beer.ex3)=c("Rating","Subject","Beer")
     head(beer.ex3)

# beer.ex and beer.ex3 are the same (though the sequences of columns 
# differ and the subject and beer labels differ). To see this:
	head(cbind(beer.ex, beer.ex3))
     
# We  clear out the beer:
	rm(Beer, beer.ex, beer.ex2, beer.ex3, beer_1, beer_2, beer1.aov, 
		beer2.aov,Rating,Subject)

### Creating a Latin Square
# The Functions.R script contains the function "Latin" 
# The parameters are n (size of square) and nrand (no. of permutations)
# The function generates a Latin square. Results will differ, depending
# on the current random number seed and value of nrand.  To test the function
     LSq=Latin(5,20)  
 

### Analysing data from a Latin square
# Read in example Latin Square data, "LSq Example.csv".
	LSq.data=read.csv(file.choose(),header=T,sep=",")
	head(LSq.data)

	S=factor(LSq.data$S)
	Time=factor(LSq.data$Time)
	Treatment=factor(LSq.data$Treatment)

# Now for the ANOVA
     summary(aov(LSq.data$Y~Treatment+Time+Error(S/Time)))
# NOTE: Because Time and Treatment are confounded within subjects,
# the results are the same if the model has Error(S/Treatment) instead
# of Error(S/Time)
     
##Exercise 13.5
# Using R, analyze the data set EX13_8  as (a) a Subjects x A design,
# and (b) a complete Latin square design.

# Clean up the workspace:
	rm(LSq,LSq.data,S,Time, Treatment)

##Exercise 13.6 
# Use the anova function to compare the two models in Exercise 13.5
# Then, remove objects created in completing the exercise
	
#################################################################
		
##### 				Session 14
#####	 One-Factor Repeated-Measures Designs

# A useful source for information about repeated measures analysis is
# http://gribblelab.org/2009/03/09/repeated-measures-anova-using-r/

### Methods for the Univariate Analysis 
# Read in the file "MaleDep.csv" (Male depression scores, age < 35)
     male_d = read.csv(file.choose(),header=T)
	head(male_d)

	season = factor(male_d$Season)
	subj=factor(male_d$Subject)

## Method 1
# The aov function provides a test of the effect of the season on the
# depression score (D_Score). As in Session 13, because there is only
@ one factor we have two options. The more versatile approach is: 
	male_d.aov = aov(D_Score~season+Error(subj/season),data=male_d)
	summary(male_d.aov)
# The effect of Seasons is significant at the 05 level,

## Method 2
# When there is only one factor, we may also use
	male_d.aov2=aov(D_Score~season+subj,data=male_d)
	summary(male_d.aov2)

# Table of effects or means
# We can get effects or means for subjects and seasons; e.g.,
     model.tables(male_d.aov2,type="means",se=T)

## NOTE: If the error term is specified in the model as in male_d.aov
# we get a message: " SEs for type 'means' are not yet implemented"

## Exercise 14.1
# Find the standard errors of the seasonal means. Plot
# the seasonal mean depression scores and their standard errors.

### The TukeyHSD Test.
# If you haven't the multcomp package in your library, install it. 
# See the script for Session 10 for other examples of the glht function.
     library(multcomp)
     summary(glht(male_d.aov2,linfct=mcp(season="Tukey")))
# The "Error" in the output = sqrt(MS_res*(2/n)); sphericity is assumed.
#  We will shortly consider contrasts tested against standard errors that
#  are based on only the means involved in the contrast.
# NOTE: "glht" does not work when the error is specified as in male_d.aov


### Nonadditivity:Tests and Transformations
# The nonadd.test function (in the Functions.R script)
# tests for nonadditivity, assumes a balanced (no missing scores) design,
# and uses the usual subjects x treatments layout rather than a stack.
# Therefore, we first have to change male_d to a 14 (subjects) x 4 (seasons
# data frame. The "reshape" function is in the Functions.R file.
	male_d2 = t(reshape(male_d[,2],4,14)); male_d2
# "reshape" takes the first 14 scores in Column 2 of male_d and places
# them in the first row of male_d2; these are the winter scores. Successive
# rows are filled with the scores for the next three seasons. The "t" 
# operator trasnposes rows and columns, We can clarify our results with
# row and column names:
	rownames(male_d2)=paste("S",1:14,sep="")
	colnames(male_d2)=c("winter","spring","summer","fall")
	male_d2

# Now for the nonadditivity test. The form of the nonadd.test function is
# nonadd.test(x,SSerror) where x is a data frame and SSerror comes
# from the ANOVA. The function is in the Functions.R script.
# From the summary output of either aov model for the male_d data, we have 
     SSerror=206.96
#Then to test the model on our male depression data
     nonadd.test(male_d2,SSerror)
# The F is for a test of nonadditivity. The relatively low p value suggests
# that the data may not be additive and a transformation might be helpful.
# The "power for transformation is" .325  Each score should be raised to that
# power. Let's see what happens when we do this.

# Let t_dep be the transformed depression score:
 	t_dep=male_d$D_Score^.325
	male_d3=cbind(male_d[,c(1,3)],t_dep)
	head(male_d3)
# Now for the reanalysis:
	male_d3.aov = aov(t_dep~season+Error(subj/season),data=male_d3)
	summary(male_d3.aov)
# Note that the p value is much smaller on the transformed scale.

# To test for nonadditivity on the transformed scale, we reshape male_d3:
	male_d4 = t(reshape(male_d3[,3],4,14))
	rownames(male_d4)=paste("S",1:14,sep="")
	colnames(male_d4)=c("winter","spring","summer","fall")
	male_d4
# Then the test is
    SSerror=6.687
     nonadd.test(male_d4,SSerror)
# Unlike the results with the original scale, there is no evidence of
# nonadditivity.

##Exercise 14.2
# Read the EX14_2.csv data set. Perform the ANOVA, then test for nonadditivity.
# What transformation is indicated?


### A Different (more complete) Analysis
# Next we work with the more common repeated-measures layout,
# in which rows are subjects and columns are levels of the 
# within-subject factor; i.e., male_d2.
     
# First form a linear model
 	model.lm =lm(male_d2 ~ 1)	#The "1" represents the intercept
     model.lm
# Note that the coefficients are the seasonal mean depression scores
# We wiah to treat the seasons as a factor.
     seasons=factor(c("Winter","Spring","Summer","Fall"))
     seasons
# To continue the analysis we need the Anova function from the car library
# Note the capital A, distinguishing Anova from anova.
# If "car" isn't yet installed, do so now.
     library(car)
     model.aov = Anova(model.lm, idata = data.frame(seasons),
       idesign = ~seasons, type="III")
     summary(model.aov, multivariate=FALSE)  #No multivariate output
# This provides the Mauchly test of sphericity, as well as results for
# the usual univariate ANOVA, and those using the Greenhouse-Geisser
# and Huynh-Feldt adjustments. Note the warning about the HF adjustment.

# Suppose we desire the multivariate tests
     summary(model.aov)
# There is considerable output but under "Multivariate Tests: seasons"
# we have the multiuvariate test results 


### Contrasts for the Repeated Measures Design
# Continuing with the seasons depression data, we note
# that the summer mean depression score seems to differ from the other
# means which are fairly similar. We could test the summer mean vs each
# other seasonal mean; e.g., summer vs winter:
     diff31=male_d2[,3]-male_d2[,1]
	t.test(diff31)
# or we could test the summer mean against the mean for the others combined.
     mean124=rowMeans(cbind(male_d2[,1],male_d2[,2],male_d2[,4]))
     t.test(mean124-male_d2[,3])
# See Script for Session 10 (Part 2) for FWE adjustment methods
     
## Exercise 14.3
# Test the polynomial components of the depression data as a function
# of seasons.

##Exercise 14.4
# Use the Anova function to analyze the data in "EX14_3.csv". Include
# pairwise tests.


### Missing Data
# Several programs have been written in R to impute (estimate) missing
# scores. These include Amelia II, MICE, and mitools. Manuals describing 
# available functions in each package, and the packages, are available 
# at the CRAN website. If you wish to discard cases with missing scores
# (NA indicates a missing score in R), na.omit does the job. 
# Consider
	x1 = c(61,54,43,59)
	x2 = c(NA,60,85,60)
	x3 = c(78,45,42,40)
	x = rbind(x1,x2,x3); x
# If we want the mean of only the cases with all 4 scores,
	rowMeans(na.omit(x))
# Only the x1 and x3 means are calculated. Now try
	rowMeans(x,na.rm=T)
# Note that na.rm permits calculations based on the available scores.

# Clean up
	rm(x1,x2,x3,x)


### Nonparametric Tests
## Friedman's Chi Square
# Again using the season depression data,
    friedman.test(male_d2,blocks=Subject)
# The result is: "Friedman chi-squared = 6.7377, df = 3, p-value = 0.08074"

## F_R, the Iman,Hora,Conover Test on Ranks (1984)
# We first rank the scores in the long form of the male depression data,
# then perform the ANOVA.
     dep.rank=rank(male_d$D_Score)
     male_d.rank =  cbind(male_d,dep.rank)
     summary(aov(dep.rank~season+subj,data=male_d.rank))
# The season result is: F = 3.4331 p = 0.02611 

## The Epsilon-Adjusted p Values and the Mauchly Test
# we need the broad layout but with ranked data.
     drm=matrix(dep.rank,nrow=14,ncol=4)
     model.r = lm(drm~1)
     seasons.r=factor(c("Winter","Spring","Summer","Fall"))
# Now load the car library to get the Anova function
     library(car)
     model.aov = Anova(model.r, idata = data.frame(seasons.r),
       idesign = ~seasons.r, type="III")
     summary(model.aov, multivariate=FALSE) 

## Wilcox Signed Rank test (WSR)
# We compare summer and spring depression medians
	male_d2=data.frame(male_d2)		# from matrix to data frame
    wilcox.test(male_d2$spring,male_d2$summer,paired=TRUE)	# p-value = 0.04131
 				
## Cochran's Q test
# A function,"cochranq.test" is in the Functions.R script (from
#  http://tolstoy.newcastle.edu.au/R/e2/help/06/09/0974.html
# The only input is a subject by conditions matrix.
# To test the function, we first create a data set
     A = c(1,1,1,1,0,0,1,0)
     B = c(1,0,0,0,0,1,0,0)
     C=c(0,0,0,0,1,0,0,1)
     qmat =cbind(A,B,C); qmat
# Now for the test
      cochranq.test(qmat)	# Cochran's Q = 2.25, df = 2, p-value = 0.3247

# Clean up the workspace
	rm(A,B,C,male_d2,diff31,drm,qmat,male_d,male_d.rank,mean124,
		model.aov,dep.rank,model.lm,model.r,seasons,
		seasons.r,SSerror,male_d.aov, male_d.aov2,season,subj)

###############################################################
#####				Session 15
##### 	Multi-Factor Repeated Measures & Mixed Designs

### SxAxB Design with A and B Fixed
# First read  "SAB Data.csv" 
     SAB0.data = read.csv(file.choose(),header=T)
     head(SAB0.data)
# We only need the first 9 columns at this time; later, 
# we will use the other columns for a polynomial analysis. For now,
	SAB.data = SAB0.data[,1:9]
 
# Now the analysis. First, model the levels as a function of the intercept
     model.lm=lm(cbind(A1B1,A2B1,A3B1,A1B2,A2B2,A3B2,A1B3,A2B3,A3B3)~1,
	data=SAB.data)
#Now establish the factor combinations.
     A=rep(c("A1","A2","A3"),3)
     B=rep(c("B1","B2","B3"),each=3)
     SAB.AB = data.frame(A=factor(A),B=factor(B))

# Now do the analysis using Anova function
     library(car)
     SAB.anova=Anova(model.lm,idata=SAB.AB,idesign=~ A*B, type="III")
     summary(SAB.anova,multivariate=FALSE)

## An alternative analysis: Using the long layout
# We can stack the data and use the aov function. Two issues: Must explicitly
#  specify the error term and output does not include the epsilon adjustments
#  or the multivariate option. This analysis requires a data frame of 4 columns
#  (Subjects,A,B,score). A and B must be factors. The model is
#   SAB.aov = aov(score~A*B+Error(Subjects/(A*B))) 

##Exercise 15.1
# Create the long layout and do the alternative analysis indicated above.

## clean up
     rm(A,B,model.lm, SAB.AB, SAB.anova) 
# Keep the SAB.data data frame; we will use it to analyze contrasts.


### Mixed (split-plot) Designs with A and B Fixed
# A is between subjects, B is within subjects
# The data set will be " Mixed Data.csv"
     mix.data=read.csv(file.choose(),header=T)[,1:5]
     head(mix.data)       # We don't need the B_linear column, so
 
## Method 1: Broad Layout
# We might want MANOVA results, or at least the epsilon-adjusted
#  univariate analysis. In any event, our data usually has subjects
#  as rows and the within-subjects levels as variables. So, we return
# to the mix.data data frame, get the car library (to access 
# the Anova function), and proceed from there:
      library(car)			# If not still loaded
      B = factor(c("B1","B2","B3","B4"))
     idata=data.frame(B)
     model.lm = lm(cbind(B1,B2,B3,B4)~factor(A),data=mix.data)
#NOTE: Because the levels of A are numeric (1,2,3), it is critical
# to designate A as a factor as in the preceding statement.
     mix.model=Anova(model.lm,idata=idata,idesign= ~B,multivariate=F)
     summary(mix.model)

## Method 2: Stacked Layout
# aov provides an analysis, but without the G=G and H-F corrections

# First, we create the long (stacked) layout
     mix.st = stack(mix.data[2:5])
     A=factor(rep(rep(c("A1","A2","A3"),each=6),4))
     B=factor(rep(c("B1","B2","B3","B4"),each=18))
     S=factor(rep(1:18,4))
     Y = mix.st[,1]
     mix2.data=data.frame(S,A,B,Y)

# The ANOVA result is obtained by
     mix2.aov= aov(Y~(A*B)+Error(S/B))
     summary(mix2.aov)

# The test results are the same for the two data layouts but the broad
# layout using the lm function provides more information.

# Clean up
	 rm(A,B,idata,mix.data,mix.model,mix.st,mix2.data,mix2.aov,model.lm,S,Y)


### Repeated Measures with a Random Factor 
# Read the data set, "FontData.csv"
# The file has 8 subjects, who read aloud six items, each presented in
# two different fonts. Time to say the word (RT) is recorded.
     font.data=read.csv(file.choose(),header=TRUE)
     head(font.data)
	attach(font.data)

# Because both subjects and items are random-effects factors,
# under the complete structural model, there is no single error
# term against which to test the font factor.One possible solution
# is to determine if a simpler model is appropriate. Specifically,
# if either the subjects x font or the items x font interaction 
# contributes negligble variance, a single mean square provides an
# error term. 

## A Preliminary ANOVA)
# To test the possibility of the simpler model,
# we first treat items as fixed, 
	subj=factor(Subject)
	font=factor(Font)
	item=factor(Item)
     font.aov = aov(RT~font*item+Error(subj/(font*item)))
     summary(font.aov)
# The F test of font:item yields F = .713, indicating that the 
# item x format variance is negligible. We can therefore revise our model
# and the EMS, and test font against subj x font, or even against
# its pool with subj:font:item.

## NOTE: When labeling sources of variance, R uses colons rather than
# asterisks. Note also that the "Residuals" in the R output
# has a different meaning depending on its position. For example,
#  the residuals under font is subj x font in the usual terminology.
#  Residuals under font:item is subj  x font x item

## Quasi_F (F.q) Tests
# To obtain quasi-F tests from the font.sov output,
# designate A as font, B as item, S as subj. Then
# taking the expected mean squares (EMS) into consideration, to test A
#  F.q = MSA/(MSAS+MSAB-MSABS)and 
# dferror = ((MSAS+MSAB-MSABS)^2)/(MSAS^2/dfas + MSAB^2/dfAB + MSABS^2/dfABS)
# From the summary(font.aov) output,
    F.q = 34846/(4622+3920-5498.6);  F.q   

# The error degrees of freedom are calculated by similar substitution

## F1, F2, and minF'
# In many journal articles, authors report F1 (test against MSAS), F2
# (test against AB), and/or minF' (f.qmin, minimum quasi-F when data are
# missing). We deleted some scores from the font data to consider these. 
# Read in "Missing Data.csv":
	font_miss = read.csv(file.choose(),header=TRUE)
	font_miss				#Note the NAs
# now to get the cell means
     A_miss=rowMeans(font_miss[,1:6],na.rm=T) # Means of columns 1 - 6
							    		 # for each subject; the A1 data
     B_miss=rowMeans(font_miss[,7:12],na.rm=T) # Means of the A2 columns
     AB_miss=stack(c(A_miss,B_miss)) 											     #These are the font means for each subject
     font=factor(c(rep("A",each=8),rep("B",each=8)))
     SxF=cbind(AB_miss,font)
     colnames(SxF)=c("RT","Subject","font")
	SxF
# The scores for each subjects are means over items for the two fonts (A and B)

# Now for the analysis of variance
     F1.aov=aov(RT~font + Error(factor(Subject)/font),data=SxF)
     summary(F1.aov)
# When tested against subject variability (items treated as having fixed 
# effects), the font factor is not significant' p=.221.

## Exercise 15.2
# (a) Using the font_miss data frame, create an items by font data frame.
# (b) Calculate F2 (font tested against font x items).
# (c) minF = F1*F2/(F1+F2). Do the calculation.
# df_error = ((F1+F2)^2)/((F1^2)/dfAB + (F2^2)/dfAS). A=font, B = items
# Do the cakculations for the font.miss data set.

## Exercise 15.3
# Calculate F1 and F2 using the original font.data.

# There are other functions that take a different approach to
#  designs with random factors. We will consider mixed-effects model
#  analyses (also called hierarchical, or multilevel) in  
# " Session 26: Mixed Effects Models"

# Clean up
	detach(font.data)
	rm(A_miss, AB_miss,B_miss,F.q,F1.aov,font,font.aov,font.data,
		font_miss,item,subj, SxF)

### Contrasts  in Multifactor Repeated Measures Designs
# One way to contrast cell means in repeated measures designs is to define
# a contrast score for each subject and perform a t test, or construct a
# confidence interval uasing those scores.
# If SAB.data is not still in your workspace, read the file "SAB Data.csv".
# In the ANOVA of these data, we found a significant AB interaction. We
# note that this may be because the B2-B3 difference is smaller at
# A1 than at either A2 or A3. Accordingly we wish to test two contrasts.
# To simplify notation:
     x=SAB.data[,1:9]
# Create the contrast scores
     psi1 = (x[,8]-x[,5])-(x[,7]-x[,4])  #(A2B3-A2B2)-(A1B3-A1B2)
     psi2 = (x[,9]-x[,6])-(x[,7]-x[,4])  #(A3B3-A3B2)-(A1B3-A1B2)
# Then to test these contrasts
     t.test(psi1)				# p = .1204
     t.test(psi2)				# p = .0044
# Assuming the contrasts were post hoc, based on an inspection of the data,
# we recommend multiplying the p values by 9,the number of possible 2 x 2
# contrasts, in accord with the Dunn-Bonferroni nethod.

## Exercise 15.4
# Using the appropriate t distriution function in R, calculate the 
# confidence interval adjusted for family size.


### Trend Analysis in Multifactor Repeated Measures Designs
# The SAB0.data frame  contains polynomial "scores" created in SPSS.
#  Here, we do the same thing using R. First, construct a matrix of
#   linear coefficients:
     L1=rep(c(-1,0,1),18)
     L = matrix(L1,nrow=6,ncol=9,byrow=T)
# Then multiply L and x
     Lx = L*x
# Sum the first 3 columns, then the next 3, then the last 3:
     LA_B1=rowSums(Lx[,1:3]); LA_B1   # linear(A) at B1
#  Compare these 6 values with the LinA_B1 column of SAB0.data
#  Now get the other linear "scores"
      LA_B2=rowSums(Lx[,4:6])
      LA_B3=rowSums(Lx[,7:9])
# we can now proceed to test the average linear function, and also
#  to test whether the slopes differ significant;y over the levels of B
#  First we need a matrix of the linear(A) scores
     LA_B0=stack(data.frame(cbind(LA_B1,LA_B2,LA_B3)))
     LinA_B = LA_B0$values   #linear(A|B)
     Cond=LA_B0$ind
     S=factor(rep(paste("S",1:6,sep=""),3))
     LA_B=data.frame(cbind(S,Cond,LinA_B))
# Now for the ANOVA
      summary(aov(LinA_B~Cond+Error(S/Cond)))

## The result for the condition (Cond), F=12.514, tells us that 
#  the three slopes differ significantly. This is the same reult 
#  found in Table 15.14 of RDASA3, where it is labeled more 
#   precisely as B x lin(A). However, this analysis does not provide
#  a test of whether the average of the three slopes differs from zero;
#  nor does it provide the three linear coefficients (slopes). So let's 
#  try something different. First, construct a subject x B 
# data frame using the linear coefficients
     LinA = data.frame(cbind(LA_B1,LA_B2,LA_B3)); LinA

     model.lm=lm(LinA~1)
     model.lm
# The model.lm output provides the means of the linear coefficients
# at each level of B

# Now let conditions be a factor:
     attach(LinA)
     Cond=factor(c("LA_B1","LA_B2","LA_B3"))
# Load the "car" library {if not still loaded) to get the Anova function
     library("car")
     model.aov = Anova(model.lm, idata =data.frame(Cond),
       idesign = ~Cond, type="III")
     summary(model.aov, multivariate=FALSE)     
# Now we have all we need. The F of 5.772, labeled (Intercept),
# tests whether the averge of the three slopes differs 
#  significantly from zero; as in the aov test, Cond tests 
#  differences among the 3 slopes.

## Exercise 15.5
# Get the 3 quadaratic coeffients of the B curves and 
# test the quad(A) and B x quad(A) sources oi variance.

## Exercise 15.6 
# In a pilot study of the effects of diet upon the ability to
# withstand physical stress, 12 volunteers were divided into
# three groups of four subjects and each given a different diet. 
# They then underwent a battery of physical tests on each of four 
# successive days. A score was obtained on each day. 
# The data are in the file "Ex15_13.csv" at my R website.
#(a) Plot the performance curves for each group.
#(b) Perform the ANOVA, including a complete trend analysis. 
# Discuss the results in terms of the plot in part (a).

## Exercise 15.7
# Two groups (A1, A2) of four subjects each are tested six times; 
# there are two levels of B and three levels of C crossing to yield
# six scores for each subject. The data are in the EX15_15 file
# at my R website.
# (a) Report the results of tests of the A and AB terms, assuming all
# factors are fixed.
#(b) Suppose C represents randomly chosen items. Again test the A and
# AB sources. How does this change the tests and the results?

# Clean up
	detach(LinA)
	rm(Cond,L,L1,LA_B,LA_B0,LA_B1,LA_B2,LA_B3,LinA,LinA_B,Lx,
		 model.aov,model.lm,psi1,psi2,S,SAB0.data, x,SAB.data)


#####       	   Session 16/17
#####	Nested and Counterbalanced Variables

### Nesting items within Factor Levels: Subjects x items/factor levels
# We have response time data  for a study in which each of 86
#  subjects categorizes 15 faces depicting three moods, 5 in each mood.
# The data are available at my R website in both the extended (univariate)
# format and the subjects x trials format. We first analyze the extended format
# in the file "NestedData2.csv".
     nested.ex = read.csv(file.choose(),header=T)
     head(nested.ex)
     attach(nested.ex)
#Let's  convert to categorical factors with more meaningful names
     Mood=factor(A)
     item=factor(B)
     S=factor(S)

## The Quasi-F Test
# For this test, first treat items as fixed.
     rt.aov=aov(Y~Mood+Mood/item + Error(S/Mood/item))
     summary(rt.aov)

# Before combining terms to obtain quasi F's and df's, some points should be
# noted. First, in the aov function: Mood/item is "item within Mood;"
# R reverses the RDASA3 within notation. Similarly, the nesting of items within
# Subject x Mood cells is represented as shown; largest block first, then next
# size, etc.
# Second, the output: The error for Mood is the subjects x mood
# interaction. Also "Mood:item" is "items within Mood".
# And its residual error term ("S:Mood:item") is computationally
# "Subjects x items within Mood."

# the F test is incorrect if we view items as a random sample
#  from a population of items. The expected mean squares dictate
# quasi_F = MS_Mood/(MS_S:Mood+MS_Mood:item-MS_S:Mood:item); i.e.,
     quasi_F = 1073858/(98662+260647-41107); quasi_F	# = 3.374768
# The numerator df are 2 as usual but the denominator df are
     df = ((98662+260647-41107)^2)/((98662^2)/14+(260647^2)/12+(41107^2/84))
     df							# = 15.87821
# and the p value is
     pf(quasi_F,2,df,lower.tail=F)		# = 06

# Clean up
	detach(nested.ex)
	rm(df,quasi_F,item,Mood,nested.ex,rt.aov,S)	

## Exercise 16.1
# Read the file, "Nested Data.csv" which has 10 Subjects (S).
# 4 conditions (A), and 6 items(I) nested in each condition. 
# Get the means and standard errors at each level of A. Then calculate
# the quasi-f test of A.


### Nesting items within Factor Levels: A Split-Plot Design
## We add a between-subjects factor to the design just considered.
# The data set to be read in is "Nested Mixed Data.csv"
     mix_nest=read.csv(file.choose(),header=T,sep=",")
    	names(mix_nest)


# Now the analysis, treating B as nested within A
     mix.aov=aov(Y~C*A+A/B+C*(A/B)+Error(C/S/A), data=mix_nest)
     summary(mix.aov)

## The output
# Assuming B random, a quasi-F can be formed from the expected mean squares.
# Note the outpouring of warnings that the variables have been converted to
# factors. We can avoid this by either -- for example --
# "A = factor(mix_nest$A) -- or by--
# "attach(mix_nested); A=factor(A)". And similarly for B, C, and S.
# If we attach the data frame, "data=mix_nest" is no needed in the model
# statement.

#  One problem is that R's notation is somewhat ambiguous. C:A represents
# the C by A interaction whereas A:B represents B nested within levels of A.
# In any event, the ouput can be used, together with knowledge of the expected
# mean squares to construct the correct F tests, including quasi-F's.

# A second point to note is that nesting is indicated in the model statement
# in the order largest to smallest unit; e.g., C/S. C/S/A. In the last
#  instance, we could have written C/S/A/B; the smallest unit is
#  implicit.

# A third point is that although we might number subjects S1 through
# S10, we obtain the same result here using the numbering S1 to S5 at both C1 
# and C2. The model's "C/S" informs us that -- e.g. -- the S1 at C1 and 
# the S1 at C2 are different individuals. 

## Exercise 16.2
# Suppose the Subjects in Exercise 16.1 were divided by gender into 2 
#  groups of 5, so that the first 120 scores were male and the remaining
#  120 scores were female_d. Add 3 to female scores and subtract 3 from male 
#  scores. Now we have a split-plot design with subjects in gender, and items
# nested within levels of A. Get the means and standard errors for the
# gender x A cells. Then perform the anovam with quasi-F tests where necessary.

##Exercise 16.3 
#The file "EX16_5.csv" contains a data set with four conditions: a sentence 
# primed by either a positive (P) or negative (N) attitude, and was either 
# relevant (R) or irrelevant (I) to the trait that was rated after reading
# the subsequent description.There are five different randomly sampled
# items (prime-description pairings) in each condition so that there are
# 20 ratings for each of the 10 subjects.Use R fubctions to test the effects.

# Clean up 
	rm(mix.aov,mix_nest)
 

### The Replicated Latin Square
# We have a Latin square with four groups (rows of the squaree corresponding  
# to assignments of treatments to time periods), with five subjects in each.
# There are four treatments, A1 - A4. The data are organized in two ways,
# columns as levels of A or as levels of B (time Periods).

# Read "LatinSqData.csv"
     rep.square=read.csv(file.choose(),header=T)
     rep.square
#  Each of 4 groups has 5 subjects; the data appear twice organized by 
# A (= treatment) and B (= Trial). For the analyses we want,
  G = rep(rep(c("g1","g2","g3","g4"),each=5),4)
# G = groups = rows of the square
     S = rep(paste("S",1:20,sep=""),4)
     A = rep(paste("A",1:4,sep=""),each=20)
     B_A1=c(rep("B4",5),rep("B3",5),rep("B1",5),rep("B2",5))
     B_A2=c(rep("B1",5),rep("B2",5),rep("B3",5),rep("B4",5))
     B_A3 = c(rep("B3",5),rep("B4",5),rep("B2",5),rep("B1",5))
     B_A4 = c(rep("B2",5),rep("B1",5),rep("B4",5),rep("B3",5))
	B = factor(c(B_A1,B_A2,B_A3,B_A4))
     lsqA=stack(rep.square[,2:5])
	Y=lsqA$values
	G=factor(G);S=factor(S);A=factor(A);B=factor(B)
#	LSq.AB = cbind(G,S,A,B,Y)
    summary(aov(Y~(G*A-B)+B+G/S+Error(G/S/A)))

#NOTE: The G:A term in the output is the between cell residual on (a-1)(a-2) df
# where a is the number of rows, columns, or treatment levels in the square.
# The Residuals on 48 df are the usual within-subject residuals.

# NOTE:The p values are based on the assumption that B has fixed effects.
# If B has random effects, G and A are tested against the between-cells 
# residual (G:A). See table 16.7 (p. 407 of RDSASA3) or Raaijmakers et al.
# (Journal of Memory and Language, 1999) for the EMS that justify this.

# Clean up
	rm(A,B,B_A1,B_A2,B_A3,B_A4,G,S,lsqA, rep.square,Y)
     

### Latin Square with a Second Grouping Factor
# "LatinSqData_2.csv" has data based on a 3 x 3 Latin Square,
#  with 12 good and 12 poor readers (C) divided among three
#  counterbalanced sequences of levels of A (context). B is the
# ordinal position of A-sub-j in the sequence (i.e., 1st, 2nd, or 3rd)
     rep.sq_2=read.csv(file.choose(),header=T)
     head(rep.sq_2)
# Create the long form with all factors included
     B_A1=rep(c(rep("B2",4),rep("B3",4),rep("B1",4)),2)
     B_A2=rep(c(rep("B1",4),rep("B2",4),rep("B3",4)),2)
     B_A3=rep(c(rep("B3",4),rep("B1",4),rep("B2",4)),2)
     B = c(B_A1,B_A2,B_A3)
     G = rep(rep(c("g1","g2","g3"),each=4),6) #Rows of the Latin squares
     S = rep(c(1:24),3)
     C=rep(rep(c("c1","c2"),each=12),3)
     lsq=stack(rep.sq_2[,6:8])
      A = lsq$ind
     Y = lsq$values
     LSq.data = data.frame(cbind(S,C,G,B,A,Y))
     S=factor(S)
     G=factor(G)
     A=factor(A)
     B=factor(B)
     C=factor(C)
     summary(aov(Y~(G*A*C-B)+B*C+C*G/S+Error(C/G/S/A)))

# C,G, and G:C should be tested against G:C:S.
# All other F ratios are correct assuming A, and B are fixed-effect
# variables. If not, F tests should be based on the EMS
# which can be found in Table 16.10 (p. 411) of RDASA3.
# Table 16.11 (p. 412) contains an error. The WSR mean square,
# 13414.17/36, should be 37.25 as is the residuals term in the R output.
# The equivalences between the R output and Table 16.11 are as follows:

#	R output  Table 16.11
#	------------------
#	G		Rows
#	G:C:S 	S/GxC
#	G:A		BCR
#	G:A:C	CxBCR

#Clean up
	rm(A,B,B_A1,B_A2,B_A3,C,G,lsq, LSq.data,S,Y,rep.sq_2)


## Exercise 16.4
# The Ex16_14 file in myu R website website contains a data set
#  for a 4 x 4 Latin square design. There are 12 subjects with
# 3 in each row (R) if the square.  A is the treatment factor and C
# is the column (or position in time) factor.  Assume C has fixed effects.
#  Perform the ANOVA.

## Exercise 16.5
#The data file "Ex17_9.csv" contains data for two groups of air controllers
# 12 under 60 years old (Age = 1) and 12 more than 60 years old (Age = 2).
# Group refers to the order of the situations, A refers to 4 situations 
# in which each subject was tested, and B refers to the position in time
#  of the test The sequence of situations was counterbalanced.Do the ANOVA.

### Balancing Carryover Effects 
#   Squares that balance first-order carry-over effects (Williams squares) 
# were generated by Oliver Sailer and can be found for 2 to 26 treatments
#  at  http://statpages.org/latinsq.html.
 

 
